/// <reference types="socket.io-client" />
/// <reference types="node" />
/// <reference types="node" />
import { EventEmitter } from 'events';
import { publicKey, SignedObject } from '@shardeum-foundation/lib-crypto-utils';
import * as Crypto from '../Crypto';
import * as NodeList from '../NodeList';
import * as State from '../State';
import * as P2P from '../P2P';
import { P2P as P2PTypes } from '@shardeum-foundation/lib-types';
export declare const socketClients: Map<string, SocketIOClientStatic['Socket']>;
export declare let combineAccountsData: {
    accounts: any[];
    receipts: any[];
};
export declare let currentConsensusRadius: number;
export declare let nodesPerConsensusGroup: number;
export declare let nodesPerEdge: number;
export declare enum DataRequestTypes {
    SUBSCRIBE = "SUBSCRIBE",
    UNSUBSCRIBE = "UNSUBSCRIBE"
}
export interface DataRequest<T extends P2PTypes.SnapshotTypes.ValidTypes> {
    type: P2PTypes.SnapshotTypes.TypeName<T>;
    lastData: P2PTypes.SnapshotTypes.TypeIndex<T>;
}
export interface CompareResponse {
    success: boolean;
    matchedCycle: number;
}
interface ArchiverTotalDataResponse {
    totalCycles: number;
    totalAccounts: number;
    totalTransactions: number;
    totalOriginalTxs: number;
    totalReceipts: number;
}
interface IncomingTimes {
    quarterDuration: number;
    startQ1: number;
    startQ2: number;
    startQ3: number;
    startQ4: number;
    end: number;
}
export type subscriptionCycleData = Omit<P2PTypes.CycleCreatorTypes.CycleData, 'certificate'> & {
    certificates: P2PTypes.CycleCreatorTypes.CycleCert[];
};
export declare function createDataRequest<T extends P2PTypes.SnapshotTypes.ValidTypes>(type: P2PTypes.SnapshotTypes.TypeName<T>, lastData: P2PTypes.SnapshotTypes.TypeIndex<T>, recipientPk: publicKey): DataRequest<T> & Crypto.TaggedMessage;
export declare function unsubscribeDataSender(publicKey: NodeList.ConsensusNodeInfo['publicKey']): Promise<void>;
export declare function initSocketClient(node: NodeList.ConsensusNodeInfo): void;
export declare function collectCycleData(cycleData: subscriptionCycleData[] | P2PTypes.CycleCreatorTypes.CycleData[], senderInfo: string, source: string): void;
export declare function clearCombinedAccountsData(): void;
export interface DataSender {
    nodeInfo: NodeList.ConsensusNodeInfo;
    types: (keyof typeof P2PTypes.SnapshotTypes.TypeNames)[];
    contactTimeout?: NodeJS.Timeout | null;
    replaceTimeout?: NodeJS.Timeout | null;
}
export declare const dataSenders: Map<NodeList.ConsensusNodeInfo['publicKey'], DataSender>;
export declare const emitter: EventEmitter;
export declare function replaceDataSender(publicKey: NodeList.ConsensusNodeInfo['publicKey']): Promise<void>;
export declare function subscribeNodeForDataTransfer(): Promise<void>;
/**
 * Sets 15s timeout
 * Removes sender from dataSenders on timeout
 * Select a new dataSender
 */
export declare function createContactTimeout(publicKey: NodeList.ConsensusNodeInfo['publicKey'], msg?: string): NodeJS.Timeout;
export declare function addDataSender(sender: DataSender): void;
export declare function getConsensusRadius(): Promise<number>;
export declare function createDataTransferConnection(newSenderInfo: NodeList.ConsensusNodeInfo): Promise<boolean>;
export declare function createNodesGroupByConsensusRadius(): Promise<void>;
export declare function subscribeConsensorsByConsensusRadius(): Promise<void>;
export declare function subscribeNodeFromThisSubset(nodeList: NodeList.ConsensusNodeInfo[]): Promise<void>;
export declare function sendDataRequest(nodeInfo: NodeList.ConsensusNodeInfo, dataRequestType: DataRequestTypes): Promise<boolean>;
export declare const clearDataSenders: () => Promise<void>;
export declare function calcIncomingTimes(record: P2PTypes.CycleCreatorTypes.CycleRecord): IncomingTimes;
export declare function joinNetwork(nodeList: NodeList.ConsensusNodeInfo[], isFirstTime: boolean): Promise<boolean>;
export declare function submitJoin(nodes: NodeList.ConsensusNodeInfo[], joinRequest: P2P.ArchiverJoinRequest & SignedObject): Promise<void>;
export declare function sendLeaveRequest(nodes: NodeList.ConsensusNodeInfo[]): Promise<void>;
export declare function sendActiveRequest(): Promise<void>;
export declare function getCycleDuration(): Promise<number>;
export declare function checkJoinStatus(activeNodes: NodeList.ConsensusNodeInfo[]): Promise<boolean>;
export declare function checkActiveStatus(): Promise<boolean>;
export declare function getTotalDataFromArchivers(): Promise<ArchiverTotalDataResponse | null>;
export declare function syncGenesisAccountsFromArchiver(): Promise<void>;
export declare function syncGenesisTransactionsFromArchiver(): Promise<void>;
export declare function syncGenesisAccountsFromConsensor(totalGenesisAccounts: number, firstConsensor: NodeList.ConsensusNodeInfo): Promise<void>;
export declare function buildNodeListFromStoredCycle(lastStoredCycle: P2PTypes.CycleCreatorTypes.CycleData): Promise<void>;
export declare function syncCyclesAndNodeList(lastStoredCycleCount?: number): Promise<void>;
export declare function syncCyclesAndNodeListV2(activeArchivers: State.ArchiverNodeInfo[], lastStoredCycleCount?: number): Promise<boolean>;
export declare function syncCyclesBetweenCycles(lastStoredCycle?: number, cycleToSyncTo?: number): Promise<boolean>;
export declare function syncReceipts(): Promise<void>;
export declare function syncReceiptsByCycle(lastStoredReceiptCycle?: number, cycleToSyncTo?: number): Promise<boolean>;
export declare const syncOriginalTxs: () => Promise<void>;
export declare const syncOriginalTxsByCycle: (lastStoredOriginalTxCycle?: number, cycleToSyncTo?: number) => Promise<void>;
export declare const syncCyclesAndTxsData: (lastStoredCycleCount?: number, lastStoredReceiptCount?: number, lastStoredOriginalTxCount?: number) => Promise<void>;
export declare const syncCyclesAndTxsDataBetweenCycles: (lastStoredCycle?: number, cycleToSyncTo?: number) => Promise<void>;
export declare function compareWithOldOriginalTxsData(lastStoredOriginalTxCycle?: number): Promise<CompareResponse>;
export declare function compareWithOldReceiptsData(lastStoredReceiptCycle?: number): Promise<CompareResponse>;
export declare function compareWithOldCyclesData(lastCycleCounter?: number): Promise<CompareResponse>;
/**
 * Syncs cycle data for a specific cycle
 * @param cycle The cycle number to sync
 * @returns True if successful, false otherwise
 */
export declare function syncCycleData(cycle: number): Promise<boolean>;
export declare function scoreCert(pubKey: string, prevMarker: P2PTypes.CycleCreatorTypes.CycleMarker): number;
export {};
