import { Sign } from './types/internalTxType';
import { DevSecurityLevel } from './types/security';
export interface CountSchema {
    count: string;
}
export declare function getRandom<T>(arr: T[], n: number): T[];
export type QueryFunction<Node, Response> = (node: Node) => PromiseLike<Response>;
export type VerifyFunction<Result> = (result: Result) => boolean;
export type EqualityFunction<Value> = (val1: Value, val2: Value) => boolean;
export type CompareFunction<Result> = (result: Result) => Comparison;
export declare enum Comparison {
    BETTER = 0,
    EQUAL = 1,
    WORSE = 2,
    ABORT = 3
}
export interface CompareQueryError<Node> {
    node: Node;
    error: string;
}
export type CompareFunctionResult<Node> = Array<CompareQueryError<Node>>;
export interface SequentialQueryError<Node> {
    node: Node;
    error: Error;
    response?: unknown;
}
export interface SequentialQueryResult<Node> {
    result: unknown;
    errors: Array<SequentialQueryError<Node>>;
}
export declare function shuffleArray<T>(array: T[]): void;
export declare const robustPromiseAll: <T>(promises: Promise<T>[]) => Promise<[T[], Error[]]>;
type TallyItem<N, R> = {
    value: R;
    count: number;
    nodes: N[];
};
export type RobustQueryResult<N, R> = {
    topResult: R;
    winningNodes: N[];
    isRobustResult: boolean;
};
export declare function robustQuery<Node = unknown, Response = unknown>(nodes: Node[], queryFn: QueryFunction<Node, Response>, equalityFn?: EqualityFunction<Response>, redundancy?: number, shuffleNodes?: boolean, // Set to true if nodes need to be shuffled before querying
delayTimeInMS?: number, // Add a delay after we have queried half of the nodes
disableFailLog?: boolean): Promise<TallyItem<Node, Response>>;
export declare function sequentialQuery<Node = unknown, Response = unknown>(nodes: Node[], queryFn: QueryFunction<Node, Response>, verifyFn?: VerifyFunction<Response>): Promise<SequentialQueryResult<Node>>;
export declare const deepCopy: <T>(obj: T) => T;
export declare const insertSorted: <T>(arr: T[], item: T, comparator?: (a: T, b: T) => number) => void;
export declare const computeMedian: (arr?: number[], sort?: boolean) => number;
export declare const binarySearch: <T>(arr: T[], el: T, comparator?: (a: T, b: T) => number) => number;
export declare function getRandomItemFromArr<T>(arr: T[], nodeRejectPercentage?: number, n?: number): T[] | undefined;
export declare function sleep(time: number): Promise<boolean>;
export declare function validateTypes(inp: object, def?: Record<string, unknown>): string;
/**
 * Checks whether the given thing is undefined
 */
export declare function isUndefined(thing: unknown): boolean;
/**
 * Attempts to execute a given asynchronous function up to a certain number of retries upon failure.
 *
 * @template T The type of the resolved value of the input function.
 * @param {() => Promise<T>} fn - The asynchronous function to execute. This function should return a Promise that resolves to a value of type `T`.
 * @param {AttemptOptions} options - Optional. Options passed to change the behavior of this function. See the `AttemptOptions` interface in this same file for details.
 * @returns {Promise<T>} A Promise that resolves to the return value of the input function, if successful.
 * @throws Will throw an error if the function fails all attempts. The error will be the last error thrown by the input function.
 *
 * Code is duplicated from shardus-global-server.
 */
export declare function attempt<T>(fn: () => Promise<T>, options?: AttemptOptions): Promise<T>;
/** A little interface to represent the options you can pass to the `attempt` function. */
export interface AttemptOptions {
    /** The maximum number of attempts to execute the function. */
    maxRetries?: number;
    /** The delay between attempts, in milliseconds. */
    delay?: number;
    /** A log prefix to prepend to error logs on each failure. */
    logPrefix?: string;
}
interface Identifiable {
    id: string | number;
}
export declare const byIdAsc: (a: Identifiable, b: Identifiable) => number;
export declare function createDirectories(pathname: string): void;
export declare function verifyMultiSigs(rawPayload: object, sigs: Sign[], allowedPubkeys: {
    [pubkey: string]: DevSecurityLevel;
}, minSigRequired: number, requiredSecurityLevel: DevSecurityLevel): {
    isValid: boolean;
    validCount: number;
};
/**
 * Generates a transaction ID (txId) for the given transaction object.
 *
 * @param tx - The transaction object. It can be an EVM transaction with a `raw` property or any other transaction object.
 * @returns The generated transaction ID as a string.
 *
 * @remarks
 * - If the transaction object has a `raw` property (indicating it is an EVM transaction), the `raw` property is used for the txId calculation.
 * - For other transaction objects, the entire object is used for the txId calculation.
 */
export declare function generateTxId(tx: any): string;
export {};
