/// <reference types="socket.io-client" />
/// <reference types="node" />
/// <reference types="node" />
import { EventEmitter } from 'events';
import * as Crypto from '../Crypto';
import * as NodeList from '../NodeList';
import { publicKey } from '@shardeum-foundation/lib-crypto-utils';
import * as State from '../State';
import { BaseModel } from 'tydb';
import { P2P as P2PTypes, StateManager } from '@shardeum-foundation/lib-types';
import type { Server } from 'socket.io';
export declare let socketServer: Server;
export declare const socketClients: Map<string, SocketIOClientStatic['Socket']>;
export interface DataRequest<T extends P2PTypes.SnapshotTypes.ValidTypes> {
    type: P2PTypes.SnapshotTypes.TypeName<T>;
    lastData: P2PTypes.SnapshotTypes.TypeIndex<T>;
}
export interface ReceiptMapQueryResponse {
    success: boolean;
    data: {
        [key: number]: StateManager.StateManagerTypes.ReceiptMapResult[];
    };
}
export interface StatsClumpQueryResponse {
    success: boolean;
    data: {
        [key: number]: StateManager.StateManagerTypes.StatsClump;
    };
}
export interface SummaryBlobQueryResponse {
    success: boolean;
    data: {
        [key: number]: StateManager.StateManagerTypes.SummaryBlob[];
    };
}
export interface DataQueryResponse {
    success: boolean;
    data: unknown;
}
export declare class ArchivedCycle extends BaseModel {
    cycleRecord: P2PTypes.CycleCreatorTypes.CycleData;
    cycleMarker: StateManager.StateMetaDataTypes.CycleMarker;
    data: StateManager.StateMetaDataTypes.StateData;
    receipt: StateManager.StateMetaDataTypes.Receipt;
    summary: StateManager.StateMetaDataTypes.Summary;
}
interface ResponseType {
    STATE_METADATA?: P2PTypes.SnapshotTypes.StateMetaData[];
    [key: string]: unknown;
}
export declare const StateMetaDataMap: Map<any, any>;
export declare let currentDataSender: string;
export declare function initSocketServer(io: Server): void;
export declare function unsubscribeDataSender(publicKey: NodeList.ConsensusNodeInfo['publicKey']): void;
export declare function initSocketClient(node: NodeList.ConsensusNodeInfo): void;
export declare function createDataRequest<T extends P2PTypes.SnapshotTypes.ValidTypes>(type: P2PTypes.SnapshotTypes.TypeName<T>, lastData: P2PTypes.SnapshotTypes.TypeIndex<T>, recipientPk: publicKey): DataRequest<T> & Crypto.TaggedMessage;
export declare function createQueryRequest(type: string, lastData: number, recipientPk: publicKey): Crypto.TaggedMessage & {
    type: string;
    lastData: number;
};
export interface DataSender {
    nodeInfo: NodeList.ConsensusNodeInfo;
    types: (keyof typeof P2PTypes.SnapshotTypes.TypeNames)[];
    contactTimeout?: NodeJS.Timeout | null;
    replaceTimeout?: NodeJS.Timeout | null;
}
export declare const dataSenders: Map<NodeList.ConsensusNodeInfo['publicKey'], DataSender>;
export declare const emitter: EventEmitter;
export declare function replaceDataSender(publicKey: NodeList.ConsensusNodeInfo['publicKey']): Promise<void>;
export declare function subscribeRandomNodeForDataTransfer(): Promise<void>;
/**
 * Sets timeout to current cycle duration + some padding
 * Removes sender from dataSenders on timeout
 * Select a new dataSender
 */
export declare function createContactTimeout(publicKey: NodeList.ConsensusNodeInfo['publicKey'], msg?: string, timeout?: number | null): NodeJS.Timeout;
export declare function createReplaceTimeout(publicKey: NodeList.ConsensusNodeInfo['publicKey']): NodeJS.Timeout;
export declare function addDataSenders(...senders: DataSender[]): void;
export declare function createDataTransferConnection(newSenderInfo: NodeList.ConsensusNodeInfo): Promise<boolean>;
export declare function sendDataRequest(sender: DataSender, dataRequest: unknown): void;
export declare function subscribeMoreConsensors(numbersToSubscribe: number): Promise<void>;
export declare function processStateMetaData(response: ResponseType): Promise<void>;
export declare function sendToExplorer(counter: number): Promise<void>;
export declare function fetchStateHashes(archivers: Node[]): Promise<unknown>;
export declare function buildNodeListFromStoredCycle(lastStoredCycle: P2PTypes.CycleCreatorTypes.CycleData): Promise<void>;
export declare function syncCyclesAndNodeList(lastStoredCycleCount?: number): Promise<void>;
export declare function compareWithOldCyclesData(archiver: State.ArchiverNodeInfo, lastCycleCounter?: number): Promise<{
    success: boolean;
    cycle: number;
}>;
export declare function syncStateMetaData(activeArchivers: State.ArchiverNodeInfo[]): Promise<boolean>;
export declare const calculateNetworkHash: (data: object) => string;
export type QueryDataResponse = ReceiptMapQueryResponse | StatsClumpQueryResponse;
export {};
