import { ArchiverReceipt, Receipt } from '../dbstore/receipts';
export interface SetGlobalTxValue {
    isInternalTx: boolean;
    internalTXType: InternalTXType;
    timestamp: number;
    from: string;
    change: {
        cycle: number;
        change: object;
    };
}
export declare enum InternalTXType {
    SetGlobalCodeBytes = 0,
    InitNetwork = 1,
    NodeReward = 2,
    ChangeConfig = 3,
    ApplyChangeConfig = 4,
    SetCertTime = 5,
    Stake = 6,
    Unstake = 7,
    InitRewardTimes = 8,
    ClaimReward = 9,
    ChangeNetworkParam = 10,
    ApplyNetworkParam = 11,
    Penalty = 12,
    TransferFromSecureAccount = 13
}
/**
 * Verifies the account hash in a global transaction receipt
 *
 * This function validates that the account hashes in a receipt match the calculated
 * hashes of the account data. It checks:
 * 1. If the receipt passes schema validation
 * 2. If the receipt is a GlobalTxReceipt, it delegates to verifyGlobalTxAccountChange
 * 3. Otherwise, it verifies:
 *    - Account IDs, before and after state hashes have matching lengths
 *    - Each account in afterStates has a matching ID from the receipt
 *    - The calculated hash of each account matches the expected hash in the receipt
 *
 * @param receipt - The transaction receipt to verify
 * @param failedReasons - Array to collect failure reasons if verification fails
 * @param nestedCounterMessages - Array to collect counter messages for metrics
 * @returns boolean - True if verification passes, false otherwise
 */
export declare const verifyGlobalTxAccountChange: (receipt: ArchiverReceipt | Receipt, failedReasons?: any[], nestedCounterMessages?: any[]) => boolean;
