import { AxiosInstance, AxiosRequestConfig } from 'axios';
import { RequestInfo, RequestInit, Response } from 'node-fetch';
/**
 * A custom fetch function with a max response body size limit.
 * @param input URL or Request object.
 * @param init Optional fetch configuration.
 * @param maxBytes Maximum response size in bytes (default: config.maxResponseSize).
 * @returns A Response.
 */
export declare function customFetch(input: RequestInfo, init?: RequestInit, maxBytes?: number): Promise<Response>;
/**
 * Creates a custom axios instance with size limiting
 * @param maxBytes Maximum response size in bytes (default: 10MB)
 * @param axiosConfig Additional axios axiosConfig options
 * @returns Custom axios instance
 */
export declare function customAxios(maxBytes?: number, axiosConfig?: AxiosRequestConfig): AxiosInstance;
